#pragma once
#include "ForwardDef.h"
#include "WMAllocator.h"

class IOContext {

};

/// A device can register a LoadFilter when it detects that some fixing will be necessary to properly load itself.
class LoadFilter {
public:
	/// You must implement Release(); if your filter is heap allocated, delete it. If your filters are stateless, it might make more sense to create them statically in the data segment
	// in which case Release() should do nothing.
	virtual void Release() = 0;
	
	// Register a filter for directly after the device is loaded. Note that the device has NOT had its world pointer set yet!
	virtual void onDeviceLoaded(Device *dev, LoadContext &iocontext) {};

	virtual void onWorldLoaded(Device *dev, LoadContext &iocontext) {};
};

/** The LoadContext is passed to your device's Load() method, and provides both additional information
about the loading environment, and also allows the device to provide feedback; for example, to notify the user
that they have loaded an old version whose functionality might not be correct any longer */
class LoadContext : public IOContext {
public:
	virtual bool isPreservingID() = 0;

	/// Returns true if this device is being loaded as a preset; if true, it means that basic things like name, location, etc will not be overwritten.
	virtual bool loadingPreset() = 0;

	/// Return the version of WM that this world was created in
	virtual int getWorldCreationAppVersion() =0;


	/// Set from the loading world to indicate the app version of the file
	virtual void setWorldFileVersion(int version) = 0;

	/// Producing a load error will stop the world loading process, so use this only in scenarios
	/// where continuing would produce nonsense/crashes. This allows you to provide a message as to why the load is being stopped.
	virtual void errorOnLoad(Device const*dev, const char *msg) = 0;

	/// Warnings on load are very useful for notifying the user if your device has changed its method of operation in a
	/// non-transparent way. The LoadContext will collect all of these messages (usually displaying them to the user after load)
	virtual void warningOnLoad(Device const*dev, const char *msg) = 0;

	/// Call to register a patch with the WM load system. See documentation for LoadPatch; but essentially this allows you
	/// to do things like programmatically add additional devices to your input/output to replicate some old functionality,
	/// or re-wire inputs
	virtual void registerFilter(Device *dev, LoadFilter *filter) = 0;

	/// Call to register a port-swap patch with the WM load system. This can even swap wirings to a new device that has been dynamically created!
	virtual void registerPatch(PortInfoStruc const&requestedport, PortInfoStruc const&remappedport) = 0;


	virtual void processFiltersDeviceLoaded(Device *dev) = 0;
	virtual void processLinkPatch( Device * &src, int &sp, int &st ) = 0;
	virtual void processFiltersWorldLoaded() = 0;

};

class SaveContext : public IOContext {
public:
	/// Returns if, and what the version code of the version we're attempting to save for. 
	virtual bool saveForOldVersion(int &out_version) = 0;
};